import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.Vector;

public class AssociationList
{
    private Vector list;

    public AssociationList()
    {
        list = new Vector();
    }

    public boolean contains( Association assoc )
    {
        return list.contains( assoc );
    }

    public void add( Association assoc )
    {
        list.addElement( assoc );
    }

    public void add( String key, String value )
    {
	list.addElement(  new Association( key, value ) );
    }
    
    public boolean containsKey( String key )
    {
        for ( int i = 0; i < list.size(); i++ )
        {
            Association assoc = (Association) list.elementAt( i );
            if ( assoc.hasKey( key ) )
                return true;
        }
        return false;
    }
    
    public Vector lookup( String key )
    {
        Vector answer = new Vector();
        for ( int i = 0; i < list.size(); i++ )
        {
            Association assoc = (Association) list.elementAt( i );
            if ( assoc.hasKey( key ) )
                answer.addElement( assoc );
        }
        return answer;
    }

    public Vector findAll( String str )
    {
        Vector answer = new Vector();
        for ( int i = 0; i < list.size(); i++ )
        {
            Association assoc = (Association) list.elementAt( i );
            if ( assoc.contains( str ) )
                answer.addElement( assoc );
        }
        return answer;
    }

    public void remove( Association assoc )
    {
        while ( list.contains( assoc ) )
            list.removeElement( assoc );
    }

    public void update( String key, String newValue )
    {
        for ( int i = 0; i < list.size(); i++ )
        {
            Association assoc = (Association) list.elementAt( i );
            if ( assoc.hasKey( key ) )
                assoc.appendValue( newValue );
        }
    }

    // -------------------------------------------------------------

    public int size()
    {
        return list.size();
    }

    public void clear()
    {
        list = new Vector();
    }

    public void saveToFile( String filename )
    {
        try {
            PrintWriter outputFile = new PrintWriter(
                                         new FileWriter( filename ) );
            for ( int i = 0; i < list.size(); i++ )
            {
                Association assoc = (Association) list.elementAt( i );
                outputFile.println( assoc );
            }

            outputFile.close();
        } catch ( IOException e ) {
            System.out.println( e );
        }
    }

    // -------------------------------------------------------------

    public void loadFromFile( InputStream source )
    {
        try {
            BufferedReader inputFile = new BufferedReader(
                                           new InputStreamReader(source) );
            String buffer = null;
            while ( true )
            {
                buffer = inputFile.readLine();
                if ( buffer == null ) break;

                int    split = buffer.indexOf( "]:[" );
                String key   = buffer.substring( 1, split );
                String value = buffer.substring( split+3, buffer.length()-1 );
                add( key, value );
            }

            inputFile.close();
        } catch ( IOException e ) {
            System.out.println( e );
        }
    }

    public void loadFromFile( String filename )
    {
        try {
            loadFromFile( new FileInputStream(filename) );
        } catch ( IOException e ) {
            System.out.println( e );
        }
    }

    // -------------------------------------------------------------

    public void loadFromFileSorted( InputStream source )
    {
        int originalSize = size();
        try {
            BufferedReader inputFile = new BufferedReader(
                                           new InputStreamReader(source) );
            String buffer = null;
            while ( true )
            {
                buffer = inputFile.readLine();
                if ( buffer == null ) break;

                int    split = buffer.indexOf( "]:[" );
                String key   = buffer.substring( 1, split );
                String value = buffer.substring( split+3, buffer.length()-1 );
                Association assoc = new Association( key, value );

                insertStartingAt( assoc, originalSize );
            }

            inputFile.close();
        } catch ( IOException e ) {
            System.out.println( e );
        }
    }

    public void loadFromFileSorted( String filename )
    {
        try {
            loadFromFileSorted( new FileInputStream(filename) );
        } catch ( IOException e ) {
            System.out.println( e );
        }
    }

    private void insertStartingAt( Association assoc, int originalSize )
    {
        for ( int i = originalSize; i < list.size(); i++ )
        {
            Association existingAssoc = (Association) list.elementAt( i );
            if ( assoc.lessThan( existingAssoc ) )
            {
                list.add( i, assoc );
                return;
            }
        }
        list.addElement( assoc );
    }

    // -------------------------------------------------------------
}
