import junit.framework.*;

public class AssociationTest extends TestCase
{
    private Association association;
    
    protected void setUp()
    {
	association = new Association( "note", "text" );
    }

    public void testToString()
    {
	assertEquals("[note]:[text]", association.toString());
    }
    
    public void testKeyContains()
    {
	assertEquals(true, association.keyContains("note"));
    }

    public void testKeyContainsSmallerString()
    {
	assertEquals(true, association.keyContains("ote"));
    }
    
    public void testKeyContainsNoString()
    {
	assertEquals(false, association.keyContains("test"));
    }
    
    public void testValueContains()
    {
	assertEquals(true, association.valueContains("text"));
    }
    
    public void testValueContainsSmallerString()
    {
	assertEquals(true, association.valueContains("ext"));
    }
    
    public void testValueContainsNoString()
    {
	assertEquals(false, association.valueContains("test"));
    }
    
    public void testContains()
    {
	assertEquals(true, association.contains("text"));
    }
    
    public void testContainsNothing()
    {
	assertEquals(false, association.contains("test"));
    }
    
    public void testHasKey()
    {
	assertEquals(true, association.hasKey("note"));
    }
    
    public void testEquals()
    {
	Association assn2 = new Association("note", "text");
	assertEquals(true, association.equals(assn2));
    }

    // ----------------------------------------------------------------

    public void testAssociationLessThanOnKey()
    {
        Association assoc1 = new Association( "abc", "123" );
        Association assoc2 = new Association( "bcd", "123" );
        assertTrue( assoc1.lessThan( assoc2 ) );
    }

    public void testAssociationLessThanOnValue()
    {
        Association assoc1 = new Association( "abc", "123" );
        Association assoc2 = new Association( "abc", "234" );
        assertTrue( assoc1.lessThan( assoc2 ) );
    }

    public void testAssociationNotLessThanOnKey()
    {
        Association assoc1 = new Association( "def", "123" );
        Association assoc2 = new Association( "abc", "234" );
        assertFalse( assoc1.lessThan( assoc2 ) );
    }

    public void testAssociationNotLessThanOnValue()
    {
        Association assoc1 = new Association( "abc", "234" );
        Association assoc2 = new Association( "abc", "123" );
        assertFalse( assoc1.lessThan( assoc2 ) );
    }

    public void testAssociationNotLessThanOnEqualObjects()
    {
        Association assoc1 = new Association( "abc", "123" );
        Association assoc2 = new Association( "abc", "123" );
        assertFalse( assoc1.lessThan( assoc2 ) );
    }

    // ----------------------------------------------------------------

    public static Test suite()
    {
        return new TestSuite( AssociationTest.class );
    }

    public static void main( String[] args )
    {
        junit.textui.TestRunner.run( suite() );
    }
}
